"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.createRouter = void 0;
const express_1 = __importDefault(require("express"));
const path_1 = __importDefault(require("path"));
const fs_1 = __importDefault(require("fs"));
const pluginArgs_1 = require("../data-service/pluginArgs");
const cors_1 = __importDefault(require("cors"));
const async_lock_1 = __importDefault(require("async-lock"));
const axios_1 = __importDefault(require("axios"));
const config_1 = require("../config");
const grid_1 = __importDefault(require("./routers/grid"));
let dashboardPluginUrl = null;
const ASYNC_LOCK = new async_lock_1.default();
const router = express_1.default.Router(), apiRouter = express_1.default.Router(), staticFilesRouter = express_1.default.Router();
router.use((0, cors_1.default)());
apiRouter.use((0, cors_1.default)());
staticFilesRouter.use((0, cors_1.default)());
/**
 * Middleware to check if the appium-dashboard plugin is installed
 * If the plugin is runnig, then we should enable the react app to
 * open the dashboard link upon clicking the device card in the UI.
 */
//TODO: Remove the middleware after integrating with dashbaod
apiRouter.use((req, res, next) => __awaiter(void 0, void 0, void 0, function* () {
    yield ASYNC_LOCK.acquire('dashboard-plugin-check', () => __awaiter(void 0, void 0, void 0, function* () {
        if (dashboardPluginUrl == null) {
            const pingurl = `${req.protocol}://${req.get('host')}/dashboard/api/ping`;
            try {
                const response = yield axios_1.default.get(pingurl);
                if (response.data['pong']) {
                    dashboardPluginUrl = `${req.protocol}://${req.get('host')}/dashboard`;
                }
                else {
                    dashboardPluginUrl = '';
                }
            }
            catch (err) {
                dashboardPluginUrl = '';
            }
        }
    }));
    req['dashboard-plugin-url'] = dashboardPluginUrl;
    return next();
}));
apiRouter.get('/cliArgs', (req, res) => __awaiter(void 0, void 0, void 0, function* () {
    res.json(yield (0, pluginArgs_1.getCLIArgs)());
}));
if (fs_1.default.existsSync(path_1.default.join(__dirname, 'public')) &&
    fs_1.default.statSync(path_1.default.join(__dirname, 'public')).isDirectory()) {
    staticFilesRouter.use(express_1.default.static(path_1.default.join(__dirname, 'public')));
}
else {
    staticFilesRouter.use(express_1.default.static(path_1.default.join(__dirname, '..', '..', 'public')));
}
router.use('/api', apiRouter);
router.use('/assets', express_1.default.static(config_1.config.sessionAssetsPath));
router.use(staticFilesRouter);
function createRouter(pluginArgs) {
    grid_1.default.register(apiRouter, pluginArgs);
    return router;
}
exports.createRouter = createRouter;
